# -*- coding: utf-8 -*-

"""
The module contains global settings and variables for the application.
"""

from __future__ import absolute_import, division, print_function

from .environ import (
    base_dir, pod_dir, pkgs_dir, logs_dir, conf_dir, data_dir, mods_dir)

from .config import settings
