# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function, unicode_literals
"""
Intended to provide the definitions of significant signals.
"""

GREENLET_CREATED = 'greenlet.created'

AGENT_STARTED = "agent.started"
AGENT_STOPPING = "agent.stopping"

MODULE_LOADED = "module.loaded"
MODULE_UNLOADED = "module.unloaded"
