body, html  { height: 100%; }html, body, div, span, applet, object, iframe,/*h1,2, h3, h4, h5, h6,*/ p, blockquote, pre,a, abbr, acronym, address,del, dfn, em, font, img, ins, kbd, q, s, samp,small, strike, strong, sub, sup, tt, var,b, u, i, center,dl, dt, dd, ol, ul, li,fieldset, form, label, legend,table, caption, tbody, tfoot, thead, tr, th, td {margin: 0;padding: 0;border: 0;outline: 0;font-size: 100%;vertical-align: baseline;background: transparent;}body { line-height: 1; }ol, ul { list-style: none; }blockquote, q { quotes: none; }blockquote:before, blockquote:after, q:before, q:after { content: ''; content: none; }:focus { outline: 0; }del { text-decoration: line-through; }table {border-spacing: 0; }

.clear {clear: both;display: block;overflow: hidden;visibility: hidden;width: 0;height: 0;}

/* Page Style */
body{
	color: #e9e2ee;
	font-size: 14px;
	font-family: sans-serif;
	line-height: 20px;
	text-shadow: 0 1px 1px rgba(0,0,0,0.75);
	background: #55575c url(bg.jpg);
	-webkit-box-shadow: inset 0 0 300px rgba(0,0,0,0.5);
	   -moz-box-shadow: inset 0 0 300px rgba(0,0,0,0.5);
			box-shadow: inset 0 0 300px rgba(0,0,0,0.5);
}
a,
a:link,
a:visited {
	color: #c6eaf7;
	font-weight: bold;
	text-decoration:none;
}
a:active,
a:hover {
	color: #d8f3fd;
	text-shadow: 0 1px 1px rgba(0,0,0,0.75), 0 0 5px rgba(198,234,247,0.4);
}
p{
	margin-bottom: 0.3em;
}
.center,
img.center {
	text-align: center;
	clear: both;
	display: block;
	margin-left: auto;
	margin-right: auto;
}
#coming-soon{
	background: rgba(0,0,0,0.0);
	display: block;
	position: absolute;
	width: 800px;
	height: 280px;
	top: 50%;
	left: 50%;
	margin: -160px 0 0 -400px;
}
#coming-soon h1{
	text-align: center;
	font-size: 35px;
}
#coming-soon img{
    clear: both;
    display: block;
    margin-left: auto;
    margin-right: auto;
    margin-bottom: 20px;
}
#coming-soon p{
	text-align: center;
}


